package me.mrletsplay.nojs.page.layout;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.Collection;
import java.util.Collections;
import java.util.List;

import me.mrletsplay.simplehttpserver.dom.html.HtmlElement;

public class GridLayout implements Layout {

	private List<String> columns;
	private List<String> rows;
	private String gap;
	private Flow flow;
	private HAlignment hAlignment;
	private VAlignment vAlignment;

	public GridLayout() {
		this.columns = new ArrayList<>();
		this.rows = new ArrayList<>();
		this.flow = Flow.ROW;
		this.hAlignment = HAlignment.NORMAL;
		this.vAlignment = VAlignment.NORMAL;
	}

	public GridLayout columns(Collection<String> columns) {
		this.columns = new ArrayList<>(columns == null ? Collections.emptyList() : columns);
		return this;
	}

	public GridLayout rows(Collection<String> rows) {
		this.rows = new ArrayList<>(rows == null ? Collections.emptyList() : rows);
		return this;
	}

	public GridLayout columns(String... columns) {
		return columns(columns == null ? null : Arrays.asList(columns));
	}

	public GridLayout rows(String... rows) {
		return rows(rows == null ? null : Arrays.asList(rows));
	}

	public GridLayout gap(String gap) {
		this.gap = gap;
		return this;
	}

	public GridLayout flow(Flow flow) {
		this.flow = flow;
		return this;
	}

	public GridLayout hAlignment(HAlignment hAlignment) {
		this.hAlignment = hAlignment != null ? hAlignment : HAlignment.NORMAL;
		return this;
	}

	public GridLayout vAlignment(VAlignment vAlignment) {
		this.vAlignment = vAlignment != null ? vAlignment : VAlignment.NORMAL;
		return this;
	}

	@Override
	public void apply(HtmlElement html) {
		String css = "display:grid;";

		if(!columns.isEmpty()) {
			css += "grid-template-columns:" + String.join(" ", columns) + ";";
		}

		if(!rows.isEmpty()) {
			css += "grid-template-rows:" + String.join(" ", rows) + ";";
		}

		if(gap != null) {
			css += "gap:" + gap + ";";
		}

		if(flow != Flow.ROW) {
			css += "grid-auto-flow:" + flow.name().toLowerCase() + ";";
		}

		if(hAlignment != HAlignment.NORMAL) {
			css += "justify-content:" + hAlignment.name().toLowerCase() + ";";
		}

		if(vAlignment != VAlignment.NORMAL) {
			css += "align-items:" + vAlignment.name().toLowerCase() + ";";
		}

		html.appendAttribute("style", css);
	}

	public static GridLayout ofColumns(int columns) {
		List<String> colSizes = new ArrayList<>();
		for(int i = 0; i < columns; i++) {
			colSizes.add("auto");
		}

		return new GridLayout()
			.columns(colSizes);
	}

	public static GridLayout ofRows(int rows) {
		List<String> rowSizes = new ArrayList<>();
		for(int i = 0; i < rows; i++) {
			rowSizes.add("auto");
		}

		return new GridLayout()
			.flow(Flow.COLUMN)
			.rows(rowSizes);
	}

	public static GridLayout ofSize(int columns, int rows) {
		List<String> colsAndRows = new ArrayList<>();
		for(int i = 0; i < Math.max(columns, rows); i++) {
			colsAndRows.add("auto");
		}

		return new GridLayout()
			.columns(colsAndRows.subList(0, columns))
			.rows(colsAndRows.subList(0, rows));
	}

	public static enum Flow {

		ROW,
		COLUMN,
		;

	}

	public static enum HAlignment {

		NORMAL,
		START,
		CENTER,
		END,
		STRETCH,
		;

	}

	public static enum VAlignment {

		NORMAL,
		START,
		CENTER,
		END,
		STRETCH,
		;

	}


}
